#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.IDENTIFICATION  informix_dbi.pl
#.LANGUAGE	  perl script (perl 5 + DBI/DBD::Informix)
#
#.PURPOSE	  Database Interface to Informix.
#
#                 &dbi_connect( $user, $pswd, $server, $database, \$dbproc );
#                 &dbi_dosql( $dbproc, \$sth, "SQL command");
#                 %row = &dbi_nextrow( $sth, @titles );
#                 $count = &dbi_countrows( $dbproc, "SQL command", @titles );
#                 &dbi_disconnect( $dbproc, \$sth );
#                 $char = &dbi_get_stringmatch_operator;
#                 $string = &dbi_create_case_insensitive_string($string);
#                 $char = &dbi_get_wildcard_operator;
#                 $char = &dbi_get_null_comparator;
#                 $null = &dbi_get_null_assignment;
#                 $flag = &dbi_can_use_parentheses;
#                 $flag = &dbi_can_update_serial_field;
#                 $new_data = &dbi_escape_characters($old_data);
#                 $char = &dbi_get_serial_field_placeholder;
#                 $key_value = &dbi_get_generated_key($dbproc, $table, $field);
#                 $password = dbi_getpassword;
#
#.AUTHOR          Informix version Curtis Wong <curtis@crl.com>
#.AUTHOR          Bo Frese Rasmussen [ST-ECF]   <bfrasmus@eso.org>
#.MODIFICATIONS   Jeff Rowe                     <beowulf@lava.net>
#
#.VERSION       1.0     22/12-1994     Creation
#                       16/01-1995     Informix version -ckw
#                       10/02-1995     make change suggested by Bo
#               1.1     26/11-1996     Use DBD::Informix and mod_perl.
#               1.2     23/09-1997     Added wildcard and null handlers.
#------------------------------------------------------------------------------	
use strict;
use DBI;
use vars qw(@i_fields);
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE	Connects to a database.
#
#.REMARKS	This function must be called before any of the other functions
#		in this package.
#		It logs in to the given database server and connects to a
#		database.
#               $dbproc MUST be passed in by reference, i.e.:
#               &dbi_connect( $user, $pswd, $server, $database, \$db_query);
#
#.RETURNS	Either an error message or a NULL value on success.
#------------------------------------------------------------------------------
sub dbi_connect
{
    my($user, $pswd, $server, $database, $dbproc) = @_;
    my($drh, $result);

    $result = "";

    $drh = DBI->install_driver( 'Informix' );
    if (!$drh) {
      $result = "Cannot install database driver: $DBI::errstr\n";
    } else {
       $$dbproc = $drh->connect( $database, 'dbuser', 'dbpass' );
       if (!$$dbproc) {
          $result = "Cannot connect to database: $DBI::errstr\n";
       } else {
          $$dbproc->{ChopBlanks} = 1;
       }
    }

    return $result;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE	Prepares and executes an SQL statement, with error check etc.
#
#.RETURNS	Either an error message or a NULL value on success.
#------------------------------------------------------------------------------
sub dbi_dosql 
{
    my($dbproc, $sth, $sql) = @_;
    my $result = "";

    #print "SQL: $sql<br>\n";
    $$sth = $dbproc->prepare($sql);

    if (!$$sth) { 
       $result = "Cannot prepare handle: $DBI::errstr\n"; 
    } elsif (!$$sth->execute) {
       $result = "Cannot execute SQL: $DBI::errstr\n";
    }

    return $result;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE	Gets the next row from a previous select (dbi_dosql).
#
#.REMARKS	After a dbi_dosql("select ... ") call, this function can be 
#		called repeatedly to retrieve all rows of the query.
#
#		Example :
#			&dbi_dosql("select * from mytab");
#			while( %row = &dbi_nextrow  ) {
#			    print $row{'columnname'};
#			    ...
#			}
#
#.RETURNS	An associative array (keyed on the column name) of formatted 
#		data, based on the datatype of the corresponding columns.
#               Return the empty list if the fetch failed.
#------------------------------------------------------------------------------
sub dbi_nextrow
{
    my($sth, @titles) = @_;
    my(@trow,$i,%row);

    if (@trow = $sth->fetchrow) { #found a row of data
    	for ( $i = 0 ; $i <= $#titles; ++$i) {
	   #print "Title: $titles[$i], Field: $trow[$i]<BR>\n";
 	   $row{$titles[$i]} = $trow[$i];
      	}
     } else {
	%row = ();     
     }
     return %row;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.PURPOSE       Returns a count of records returned by a SQL statement.
#
#.REMARKS       Requires a SQL statement in the form:
#               "select count(*) from tablename where where_clause"
#
#.RETURNS       The number of rows matching the query.
#------------------------------------------------------------------------------
sub dbi_countrows
{
    my($dbproc, $sql, @dummy) = @_;
    my($ch, @titles, %count_result);

    &dbi_dosql($dbproc, \$ch, $sql);
    push(@titles, "(count(*))");
    %count_result = &dbi_nextrow($ch, @titles);
    $ch->finish;
    undef $ch;
    return $count_result{'(count(*))'};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.PURPOSE	Disconnects the current database connection
#
#.REMARKS	After this function is called there are no current database 
#		connection => no other function from this package can be 
#		called before a new dbi_connect call.
#
#.RETURNS	nothing.
#------------------------------------------------------------------------------
sub dbi_disconnect
{
   my($dbproc, $sth) = @_;

   if ($$sth) {
      $$sth->finish;
      undef $$sth;
   }
   if ($dbproc) {
      $dbproc->disconnect;
   }
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#		Stubbed out in Isqlperl version - ckw
#.PURPOSE	Sets the maximim number of rows returned from a query.
#
#.REMARKS	Causes SQL server to stop processing the query ( select, 
#		insert update, delete) after the specified number of rows 
#		are affected.
#		
#		PORTING NOTE: This function is added for efficiency only. 
#		When used with WDB it can safely be ignored ( leave an
#		empty function body : {} )
#
#.RETURNS	nothing.
#------------------------------------------------------------------------------
sub dbi_rowcount
{
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       Returns the database-specific term used for substring matches.
#------------------------------------------------------------------------------
sub dbi_get_stringmatch_operator
{
   return "matches";
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       Returns a case-insensitive search string.
#------------------------------------------------------------------------------
sub dbi_create_case_insensitive_string
{
   my($val) = @_;

   $val  =~ s/([a-zA-Z])/[\U$1\L$1]/g;
   return "$val";
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       Returns the database-specific character for wildcard matches.
#------------------------------------------------------------------------------
sub dbi_get_wildcard_operator
{
   return "*";
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       Returns the database-specific character for comparing a value
#               to NULL.
#------------------------------------------------------------------------------
sub dbi_get_null_comparator
{
   return " is ";
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       Returns the database-specific method for assigning NULL to a
#               value.
#------------------------------------------------------------------------------
sub dbi_get_null_assignment
{
   return "NULL";
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       Some databases support parentheses in the 'where' clause to
#               group query parameters, and some don't.  This tells WDB if it
#               can use them.
#------------------------------------------------------------------------------
sub dbi_can_use_parentheses
{
   return 1;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       Some databases don't allow you to update a field of serial data
#               type.  This tells WDB if it can update it.
#------------------------------------------------------------------------------
sub dbi_can_update_serial_field
{
   return 0;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       Escapes special characters in data using whatever method is
#               preferred by a database.
#
#.REMARKS       You could also do things like remove leading and trailing
#               blanks, etc.
#------------------------------------------------------------------------------
sub dbi_escape_characters
{
   my($data) = @_;

   $data =~ s/'/''/g;
   return $data;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#		Use informix env variable
#		Stubbed out in Isqlperl version - ckw
#.PURPOSE	Sets the default output format for dates.
#
#.REMARKS	Sets the order of the date parts month/day/year for entering
#		datetime data. Valid arguments are mdy, dmy, ymd, ydm, myd, dym.
#
#.RETURNS	nothing.
#------------------------------------------------------------------------------
sub dbi_dateformat
{
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       Gets the next serial key value for a particular table.
#
#.REMARKS       Not needed for Informix.
#
#.RETURNS       Nothing for Informix.
#------------------------------------------------------------------------------
sub dbi_getnextserialvalue
{
   return 0;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       Returns the value used by this database as a placeholder in a
#               serial type field during an insert.
#
#.RETURNS       A zero for Informix.
#------------------------------------------------------------------------------
sub dbi_get_serial_field_placeholder
{
   return 0;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       After data is inserted into a serial type key field, get the
#               value generated by the database.
#
#.RETURNS       The new serial field value.
#------------------------------------------------------------------------------
sub dbi_get_generated_key
{
    my($sth, $dbproc) = @_;
    my($value) = 0;
    my(@sqlerrd) = (0,0,0,0,0,0,0,0,0);

   if ($dbproc) {
      @sqlerrd = @{$dbproc->{ix_sqlerrd}} ;
      $value = $sqlerrd[1];
      #print "<BR>$value<BR>\n";
   }

    return $value;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       Different databases have different syntx for 'not equal', such
#               as '!=' or '<>';
#
#.RETURNS       The correct not equal syntax.
#------------------------------------------------------------------------------
sub dbi_get_not_equal
{
    return "!=";
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#.PURPOSE       Gets the password for a specified user.
#
#.REMARKS       Requires a table named security with username and password
#               fields.
#                 Table : security
#                 Fields: username, password
#
#               NOTE: VERY insecure.  No password encryption used.  So it's a
#               kludge.  No one without database access can get in anyway.  :)
#               Also, passwords must be inserted into the database beforehand.
#
#               Feel free to substitute your own code instead.
#
#.RETURNS       The password.
#------------------------------------------------------------------------------
sub dbi_getpassword
{
    my($dbproc, $username) = @_;
    my($sth, $sql, $password, @titles, %row);

    $sql = "select password from security where username = '$username'";

    &dbi_dosql($dbproc, \$sth, $sql);
    @titles = qw(password);

    %row = &dbi_nextrow($sth, @titles);
    $password = $row{'password'};
    $sth->finish;
    undef $sth;

    return $password;
}

1;
